'' =================================================================================================
''
''   File....... jm_dual_pwm_demo.spin
''   Purpose....  
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015-2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 14 MAR 2016 
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
                                                                  
  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                  
                                                                  
con { pab io pins }                                               
                                                                  
  RX1     = 31                                                  ' programming / terminal
  TX1     = 30                                                   
                                                                 
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28                                                   
                                                                 
  AUD_RT  = 27                                                  ' audio (d/a) outputs
  AUD_LF  = 26                                                   
                                                                 
  LED_R   = 27                                                  ' leds (on audio outputs)
  LED_L   = 26                                                   
                                                                 
  SD_CS   = 25                                                  ' usd card
  SD_DI   = 24                                                   
  SD_CLK  = 23                                                   
  SD_DO   = 22                                                   
                                                                 
  ADC_CS  = 21                                                  ' adc input
  ADC_CLK = 20                                                   
  ADC_DO  = 19 { miso }                                          
  ADC_DI  = 18 { mosi }                                          
                                                                 
  SVO_6   = 17                                                  ' servo headers
  SVO_5   = 16                                                   
                                                                 
  SVO_4   = 15                                                   
  SVO_3   = 14                                                   
                                                                 
  SVO_R   = 13
  SVO_L   = 12
    

con { initial pwm settings }

  PWM_FREQ = 25_000

  PWM2     = 27 
  PWM1     = 26
  

obj

' main                                                          ' * master spin cog
  time   : "jm_time"                                            '   delays/timing
  io     : "jm_io"                                              '   basic io
  pwm    : "jm_dual_pwm_ez"                                     ' * dual pwm driver
  term   : "jm_fullduplexserial"                                ' * serial for terminal
  parser : "jm_parser"                                          '   text parsing engine
  str    : "jm_strings"                                         '   strings processing  
                                                          
' * uses cog when loaded                                          
  

var


dat { valid characters and tokens }

  CHAR_SET              byte    "$%+-_."
                        byte    "0123456789"
                        byte    "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0   
  
  TOKEN_LIST
  TOKEN_FREQ            byte    "FREQ", 0                       ' FREQ { frequency in hz }
  TOKEN_DUTY            byte    "DUTY", 0                       ' DUTY { channel [ 1 or 2 ] } { percent }
                                                                 
                                                                 
' Usage examples:
' -- FREQ 20000   ... set pwm frequency to 20kHz
' -- DUTY 0 100   ... set duty cycle for pwm #0 to 100.0% 
' -- DUTY 1 51.3  ... set duty cycle for pwm #1 to  51.3%
  

con { enumerated token indexes }

  #0, T_FREQ, T_DUTY
                                                                 
  TOKEN_COUNT = T_DUTY+1                                        ' total # of known tokens
                                                                 
                                                                 
pub main | c, ready                                              
                                                                 
  setup                                                          
                                                                 
  repeat                                                         
    c := term.rx                                                ' get character from stream
    ready := parser.enqueue(c)                                  ' put into command queue
    if (ready)                                                  ' true when tokens are available
      process_cmd                                               ' process valid tokens
      parser.reset                                              ' reset parser for next command
                                                                 
                                                                 
pub process_cmd | tidx                                           
                                                                 
'' Process command tokens                                        
                                                                 
  tidx := parser.get_token_id(parser.token_addr(0))             ' get index of command token
                                                                 
  case tidx                                                      
    T_FREQ : set_freq                                            
    T_DUTY : set_duty                                            
                                                                 
                                                                 
pub set_freq | f                                                 
                                                                 
  if (parser.token_count <> 2)                                  ' check token count
    return                                                       
                                                                 
  ifnot (parser.token_is_number(1))                             ' 2nd token a number?
    return                                                       
                                                                 
  f := parser.token_value(1)                                    ' extract frequency
                                                                 
  f := pwm.freq(f)                                              ' set pwm frequency
                                                                 
  term.str(string("New frequency is "))                         ' report
  term.dec(f)                                                    
  term.str(string(" Hz"))                                        
  term.tx(term#CR)                                               
                                                                 
                                                                 
pub set_duty | ch, dc                                            
                                                                 
  if (parser.token_count <> 3)                                  ' check token count
    return                                                       
                                                                 
  ifnot (parser.token_is_number(1))                             ' 2nd token a number?
    return                                                       
                                                                 
  ch := parser.token_value(1)                                   ' convert channel token to #
  if ((ch <> 0) and (ch <> 1))                                  ' validate channel
    return                                                       
                                                                 
  ifnot (parser.token_is_number(2))                             ' 3nd token a number?
    return                                                       
                                                                 
  dc := str_to_tenths(parser.token_addr(2))                     ' convert duty token to #  

  dc := pwm.duty(ch, dc)                                        ' set duty cycle for channel

  term.str(string("New duty cycle for Ch"))                     ' report
  term.tx(ch + "0")                                              
  term.str(string(" is "))                                       
  term.dec(dc/10)
  term.tx(".")
  term.tx(dc//10 + "0")                                                   
  term.tx("%")                                                   
  term.tx(term#CR)


pub str_to_tenths(p_str) | dpidx, rnd, value

'' Convert ASCII string to number expressing 10ths

  dpidx := str.first(".", p_str)                                 ' look for decimal point

  if ((dpidx < 0) or (dpidx == strsize(p_str)-1))                ' if no dpoint or trailing digit found
    value := str.asc2dec(p_str, 10) * 10                         '   convert whole # to 10ths  

  else
    rnd := byte[p_str][dpidx+2]                                  ' get possible rounding character
    if ((rnd => "0") and (rnd =< "9"))                           ' if a digit
      rnd := (rnd - "0") / 5                                     '  convert to round-up value
    else
      rnd := 0
    byte[p_str][dpidx+2] := 0                                    ' truncate string after 10ths digit
    value := str.asc2dec(p_str, 10) + rnd                        ' convert and round up

  return value                                             
                                                                 
                                                                 
pub setup                                                        
                                                                 
  time.start                                                    ' for pause and timing
                                                                 
  pwm.start(PWM1, PWM2, PWM_FREQ)                               ' start dual pwm cog
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 
  parser.start(@CHAR_SET, @TOKEN_LIST, TOKEN_COUNT, false)      ' setup parsing engine                        
                                                                 

dat { license }

{{

  Copyright (c) 2015-2016 Jon McPhalen  

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}